// ******************************************************************
//
//    File: App_STD108_Standard.c
//    Date: 2016-04-13
// Version: 2.00
//
// Purpose:
//
// This is the standard app for TWN4 readers, which is installed
// as default app on TWN4 readers. This script can be run on any
// type of TWN4 reader without modification.
// 
// Feel free to modify this program for your specific purposes!
//
// V1:
// ------
// - Initial release
//
// V2: 
// ---
// - Support configuration via AppBlaster
//
// V1.03: 
// ------
// - Changed version numbering scheme
//
// V1.04: 
// ------
// - Configure tags being searched for only if LFTAGTYPES or
//   HFTAGTYPES requests a specific type. Otherwise, default setup
//   by firmware is used.
// - Marked as template for AppBlaster
// - Support ICLASS option
// - Change style how options are evaluated more convenient
//
// V1.05: 
// ------
// - NFCP2P Demo Support
//
// V1.06:
// ------
// - Adaption to reworked NFC SNEP service
//
// V1.07:
// ------
// - Adaption to new naming scheme of functions for host communication
// - iCLASS: Read UID by default instead of PAC
//
// V1.08:
// ------
// - Optional support for remote wakeup
//
// V2.00:
// ------
// - Adaption to new architecture of AppBlaster V2
//
// ******************************************************************

#include "twn4.sys.h"
#include "apptools.h"

// ******************************************************************
// ****** Configuration Area ****************************************
// ******************************************************************

#if APPEXTCONFIG

#include "appconfig.h"

#else

#define LFTAGTYPES        		(ALL_LFTAGS & ~(TAGMASK(LFTAG_TIRIS) | TAGMASK(LFTAG_COTAG)))
#define HFTAGTYPES        		(ALL_HFTAGS & ~(TAGMASK(HFTAG_NFCP2P)))
#define CARDTIMEOUT				2000UL	// Timeout in milliseconds
#define MAXCARDIDLEN            32		// Length in bytes
#define MAXCARDSTRINGLEN		128   	// Length W/O null-termination

bool ReadCardData(int TagType,const byte* ID,int IDBitCnt,char *CardString,int MaxCardStringLen)
{
	// Select data from card (take any ID from any transponder)
	byte CardData[32];
	int CardDataBitCnt;
    CardDataBitCnt = MIN(IDBitCnt,sizeof(CardData)*8);
    CopyBits(CardData,0,ID,0,CardDataBitCnt);

	// Modify card data (do not modify, just copy)

  	// Convert data to ASCII
    ConvertBinaryToString(CardData,0,CardDataBitCnt,CardString,16,(CardDataBitCnt+7)/8*2,MaxCardStringLen);
    return true;
}

// ****** Event Handler *********************************************

void OnStartup(void)
{
    SetVolume(30);
    BeepLow();
    BeepHigh();

    LEDOn(GREENLED);
    LEDOff(REDLED);
}

void OnNewCardFound(const char *CardString)
{
	// Send card string including prefix (actually no prefix) and suffix ("\r")
    HostWriteString(CardString);
    HostWriteString("\r");
    
    LEDOff(GREENLED);
    LEDOn(REDLED);
    LEDBlink(REDLED,500,500);

    SetVolume(100);
    BeepHigh();
}

void OnCardFound(const char *CardString)
{
}

void OnCardTimeout(const char *CardString)
{
    LEDOn(GREENLED);
    LEDOff(REDLED);
}

#endif

// ******************************************************************
// ****** Main Program Loop *****************************************
// ******************************************************************

int main(void)
{
    LEDInit(REDLED | GREENLED);
	SetTagTypes(LFTAGTYPES,HFTAGTYPES);
	char OldCardString[MAXCARDSTRINGLEN+1]; 
    OldCardString[0] = 0;
	OnStartup();    	
    while (true)
    {
		int TagType;
		int IDBitCnt;
		byte ID[MAXCARDIDLEN];
		char CardString[MAXCARDSTRINGLEN+1];
		
	    if (SearchTag(&TagType,&IDBitCnt,ID,MAXCARDIDLEN))
	    {
			if (ReadCardData(TagType,ID,IDBitCnt,CardString,MAXCARDSTRINGLEN))
			{
	    		if (strcmp(CardString,OldCardString) != 0)
	    		{
	    			strcpy(OldCardString,CardString);
	    			OnNewCardFound(CardString);
	    		}
	           	StartTimer(CARDTIMEOUT);
    			OnCardFound(CardString);
			}
	    }
		
        if (TestTimer())
        {
		    OldCardString[0] = 0;
		    OnCardTimeout(OldCardString);
        }
    }
}
