package com.handheldgroup.androdirector;

import android.os.Bundle;
import android.util.Log;
import android.widget.TextView;

import java.io.IOException;

public class GetVersionStringActivity extends SerialPortActivity {
    private static final String TAG = GetVersionStringActivity.class.getSimpleName();

    boolean mByteReceivedBack;
    final Object mByteReceivedBackSemaphore = new Object();
    String mGetVersion, mGetDeviceType, mGetDeviceUID;
    int cmd = 0;

    SendingThread mSendingThread;
    TextView mTextGetVersionString;
    TextView mTextGetDeviceType;
    TextView mTextGetDeviceUID;
    byte[] txbuffer = new byte[50];
    byte[] rxbuffer_raw = new byte[200];
    byte[] rxbuffer = new byte[200];
    int rxbuffer_Index;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_get_version_string);
        mTextGetVersionString = findViewById(R.id.TextGetVersionStringValue);
        mTextGetDeviceType = findViewById(R.id.TextGetDeviceTypeValue);
        mTextGetDeviceUID = findViewById(R.id.TextGetDeviceUIDValue);
        if (mSerialPort != null) {
            mSendingThread = new SendingThread();
            mSendingThread.start();
        }
        mTextGetVersionString.setText("");
        mTextGetDeviceType.setText("");
        mTextGetDeviceUID.setText("");
        mGetVersion = null;
        mGetDeviceType = null;
        mGetDeviceUID = null;
        cmd = 1;                                                //start with GetVersionString
    }

    @Override
    protected void onDataReceived(byte[] buffer, int size) {

        synchronized (mByteReceivedBackSemaphore) {
            mGetDeviceType = String.format("%02X", size);
            Log.d(TAG, mGetDeviceType);

            int i;
            int temp1, temp2;
            for (i = 0; i < size; i++)                    // read the bytes and save them
            {
                rxbuffer_raw[rxbuffer_Index++] = buffer[i];
            }
            for (i = 0; i < rxbuffer_Index / 2; i++)        // transfer two raw bytes to one byte
            {
                temp1 = (rxbuffer_raw[i * 2] >= 65) ? (rxbuffer_raw[i * 2] - 55) : (rxbuffer_raw[i * 2] - 48);
                temp2 = (rxbuffer_raw[i * 2 + 1] >= 65) ? (rxbuffer_raw[i * 2 + 1] - 55) : (rxbuffer_raw[i * 2 + 1] - 48);
                rxbuffer[i] = (byte) ((temp1 << 4) + temp2);
            }
            if (cmd == 2) {                                // GetVersionString
                mGetVersion = new String(rxbuffer);
            } else if (cmd == 3) {                    // GetDeviceType
                mGetDeviceType = String.format("%02X", rxbuffer[1]);
            } else if (cmd == 4) {                    // GetDeviceUID
                mGetDeviceUID = String.format("%02X%02X%02X %02X%02X%02X ", rxbuffer[1], rxbuffer[2], rxbuffer[3], rxbuffer[4], rxbuffer[5], rxbuffer[6]);
                mGetDeviceUID += String.format("%02X%02X%02X %02X%02X%02X", rxbuffer[7], rxbuffer[8], rxbuffer[9], rxbuffer[10], rxbuffer[11], rxbuffer[12]);
            }
            //mByteReceivedBack = true;
            //mByteReceivedBackSemaphore.notifyAll();
        }
    }

    @Override
    protected void onDestroy() {
        if (mSendingThread != null)
            mSendingThread.interrupt();
        super.onDestroy();
    }

    private class SendingThread extends Thread {
        @Override
        public void run() {
            while (!isInterrupted()) {
                synchronized (mByteReceivedBackSemaphore) {
                    mByteReceivedBack = false;
                    try {
                        if (mOutputStream != null) {
                            if (cmd == 1) {                    // Reset
                                rxbuffer_Index = 0;
                                txbuffer[0] = '0';
                                txbuffer[1] = '0';
                                txbuffer[2] = '0';
                                txbuffer[3] = '1';
                                txbuffer[4] = '\r';
                                mOutputStream.write(new String(txbuffer).getBytes(), 0, 5);
                                Log.d(TAG, "Reset");
                            } else if (cmd == 2) {                    // GetVersionString
                                rxbuffer_Index = 0;
                                txbuffer[0] = '0';
                                txbuffer[1] = '0';
                                txbuffer[2] = '0';
                                txbuffer[3] = '4';
                                txbuffer[4] = 'F';
                                txbuffer[5] = 'F';
                                txbuffer[6] = '\r';
                                mOutputStream.write(new String(txbuffer).getBytes(), 0, 7);
                                Log.d(TAG, "GetVersionString");
                            } else if (cmd == 3) {               // GetDeviceType
                                rxbuffer_Index = 0;
                                txbuffer[0] = '0';
                                txbuffer[1] = '0';
                                txbuffer[2] = '0';
                                txbuffer[3] = '6';
                                txbuffer[4] = '\r';
                                mOutputStream.write(new String(txbuffer).getBytes(), 0, 5);
                            } else if (cmd == 4) {               // GetDeviceUID
                                rxbuffer_Index = 0;
                                txbuffer[0] = '0';
                                txbuffer[1] = '0';
                                txbuffer[2] = '0';
                                txbuffer[3] = '8';
                                txbuffer[4] = '\r';
                                mOutputStream.write(new String(txbuffer).getBytes(), 0, 5);
                            }
                        } else {
                            return;
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                        return;
                    }
                    // Wait for 100ms before sending next byte, or as soon as
                    // the sent byte has been read back.
                    try {
                        mByteReceivedBackSemaphore.wait(250);
                        if (!mByteReceivedBack) {
                            cmd++;
                            if (cmd > 4) { //again
                                cmd = 1;
                            }
                            // Timeout
                        }
                        runOnUiThread(new Runnable() {
                            public void run() {
                                mTextGetVersionString.setText(mGetVersion);
                                mTextGetDeviceType.setText(mGetDeviceType);
                                mTextGetDeviceUID.setText(mGetDeviceUID);
                            }
                        });
                    } catch (InterruptedException ignored) {
                    }
                }
            }
        }
    }
}
